require 'socket'
require 'openssl'
require 'json'

class AMXClient
	def sock
		@sock ||= new_sock
	end

	def sock= (value)
		@sock = value
	end

	def initialize (app_id, hostname, username, password = '', keepalive = false, secure = true)
		@app_id = app_id
		@hostname = hostname
		@secure = secure
		if !password.empty? then
			@remember_token = auth username, password
		else
			@remember_token = username
		end
		@keepalive = keepalive
		exit_real unless @keepalive or password.empty?
	end

	def list (limit = '*', namespace = '/')
		auth @remember_token unless @keepalive
		value =  JSON.parse(send_message "LIST #{limit} #{namespace}")
		exit_real unless @keepalive
		return value
	end

	def send (message)
		auth @remember_token unless @keepalive
		value = send_message 'SEND', message.to_json
		exit_real unless @keepalive
		return value
	end

	def fetch (message_id)
		auth @remember_token unless @keepalive
		value = JSON.parse(send_message "FETCH #{message_id}")
		exit_real unless @keepalive
		return value
	end

	def exit ()
		return unless @keepalive
		exit_real
	end

	#PROPERTIES
	attr_reader :remember_token

	private

	def auth (username, password = '')
		send_message "APP #{@app_id}"
		token = send_message "AUTH #{username} #{password}"
		return token.chop
	end

	def exit_real
		send_message 'EXIT'
		sock.close
		self.sock = nil
	end

	def send_message (message, body = nil)
		sock.puts message
		if (body) then
			sock.puts (body)
			sock.puts '.'
		end
		response = ''
	    while line = sock.gets and line.chop != '.' do   # Read lines from the socket
	    	response += line
		end
		return response
	end

	def new_sock
		s = TCPSocket.new(@hostname, 2026)
		if @secure then
			ctx = OpenSSL::SSL::SSLContext.new
			ctx.ssl_version = :SSLv23_client
			ssl = OpenSSL::SSL::SSLSocket.new(s, ctx)
			ssl.sync_close = true
			ssl.connect
			return ssl
		else
			return s
		end
	end
end